This module will add a block with a maplibre map, To override options alter the
map with key 'maplibre_canvas_block' using the hook
maplibre_canvas_block_maplibre_gl_map_info_alter .

Ideally you should place the block together with a views page that will
provide the features (and also could present a list of these features).

# Configuration

To make it work the view that is used as index of the maplibre canvas, the one
that provides the features, to include in footer a views_area with any view
with a geojson_export display.
The usual way is to add a view with page display that shows a list of the
map elements, and in the same view a new display with the same arguments
that uses the geojson_export, this display is the one to add in page display
footer as view area.
Take care that only the footer element with machine named 'view' will be used.
Also the view_area with json will be unsetted so will not be visible.

# How uses maplibre_gl module?

This module also is an example of how maplibre_gl module could be used as base
to obtain a rich map.
In this case it plays with the alter hook to achieve marker using an html from
a field in a view output setting geojson display type view with a field named
'html_marker', if it is there and in hook_maplibre_gl_info_alter in a custom
module or in active theme is defined that point layer is of `symbol` type:

`$map_info['maplibre_canvas_block']['layers'][0]['type'] = 'symbol';`

Then the html of a field could be used, for example based on category, and using
a rich icons set.

All other options are there to be tweaked, take a look at maplibre_gl.api.php
for the altering example.


# Examples

Although this module is intended to be used with a geojson view, it also could be
used without it, so with a static geojson placed somewhere locally or remotely.

Here are some examples:

## Setting features

### Features directly from the altered array

```php
function communia_radix_maplibre_gl_map_info_alter(array &$map_info) {
  $map_info['maplibre_canvas_block']['options']['style'] = 'https://geoserveis.icgc.cat/styles/icgc_orto_hibrida.json';
  $coordinates = [2.009226, 41.5677025];

  // center & zoom
  $map_info['maplibre_canvas_block']['options']['center'] = $coordinates;
  $map_info['maplibre_canvas_block']['options']['zoom'] = 17;

  // json coordinates
  $map_info['maplibre_canvas_block']['sources']['custom-points'] = [
    'type' => 'geojson',
    'data' => [
      'type' => 'FeatureCollection',
      'features' => [
        [
          'type' => 'Feature',
          'geometry' => [
            'type' => 'Point',
            'coordinates' => $coordinates,
          ],
          'properties' => [
            'title' => 'Communia SCCL',
            'description' => 'Communia SCCL. Carrer Sant Gaietà 73, 08221, Terrassa.',
          ],
        ]
      ]
    ]
  ];
  ```
### Features directly from external static json

  Or getting the data from a json:

  ```php
function communia_radix_maplibre_gl_map_info_alter(array &$map_info) {

    // json coordinates
    $map_info['maplibre_canvas_block']['sources']['custom-points'] = [
      'type' => 'geojson',
      'data' => 'https://yoursite.coop/themes/custom/your_theme/assets/map_points.json'
    ];
  }
  ```

## Setting the layers, so how points are viewed

### Circle

Then, this source could use layers to show points as circle:

  ```php
function communia_radix_maplibre_gl_map_info_alter(array &$map_info) {
    (...)
  // Map point
  $map_info['maplibre_canvas_block']['layers'][] = [
    'id' => 'communia-point-layer',
    'name' => 'Custom points',
    'type' => 'circle',
    'source' => 'custom-points',
    'paint' => [
      'circle-radius' => 8,
      'circle-color' => '#E63946',
       'circle-stroke-width' => 2,
      'circle-stroke-color' => '#ffffff',
    ],
    'layout' => [
      'visibility' => 'visible',
    ],
  ];

```

### Custom image symbol

One could also use custom images. However it needs to load previously the image
(in this case: custom-logo). Image loading must be done listening to the
maplibre-gl-js new-layer or new-map events depending on needs, as in example
https://maplibre.org/maplibre-gl-js/docs/examples/add-image/ .

The loading will be something like:

```javascript
(function (Drupal, drupalSettings, once) {
  'use strict';
  Drupal.behaviors.mapImage = {
  attach: function (context, drupalSettings) {
    const maplibre_once = once('mapIcon', '#maplibre_canvas_block', context)
      if (!maplibre_once.length) return;
      maplibre_once.forEach((mapCanvas)=> {
        const container = mapCanvas.id
        document.getElementById(container).addEventListener(
        'new-map',  async (event) => {
          const image = await event.detail.map.loadImage('https://yoursite.coop/themes/custom/your_theme/icons/logo.png');
          event.detail.map.addImage('custom-logo', image.data);
        });
    })
  }
})(Drupal, drupalSettings, once);
```

And then use the loaded and added custom-logo.

```php
function communia_radix_maplibre_gl_map_info_alter(array &$map_info) {
  (...)
  $map_info['maplibre_canvas_block']['layers'][] = [
    'id' => 'custom-point-layer',
    'name' => 'Custom points',
    'type' => 'symbol',
    'source' => 'custom-point',
    'layout' => [
      'icon-image' => 'custom-logo',
    ],
  ];
```