(function (Drupal,once) {
  Drupal.behaviors.maplibre_canvas_block_layer_fetch = {
    attach: function (context, settings) {
      const once_elements = once('layer_fetch',".maplibre-canvas-block-geojson-data-provider", context)
      once_elements.forEach(element => {
        const targetNode = element //context.getElementsByClassName(".maplibre-canvas-block-geojson-data-provider").item(0)
        var map = {};
        var visibleMarkers = {};

        // look for maplibreGl container definitions to hang on to map events
        for (const container in settings.maplibreGl ) {
          document.getElementById(container).addEventListener(
            'maplibre-gl.map', (event) => {
              map = event.detail.map;
              if (!map.getSprite()) {
                map.setSprite('https://demotiles.maplibre.org/styles/osm-bright-gl-style/style.json');
              }
              addPoints();
              map.on('sourcedata', function (e) {
                // Do things when map events fire.
                if (e.sourceId !== 'points' || !e.isSourceLoaded) return
                // var f = map.querySourceFeatures('points')
                if (e.source.data.features.length === 0) return
                // Only add markers if no symbol type and icon-image defined on points layer
                //if (map.getLayer('points').type == 'symbol' && !map.getLayer('points').getLayoutProperty('icon-image')) addMarkers(e.source.data);
              })
              map.on('moveend', function(e){
                // Do things whenever map move ends
                if (map.getLayer('points').type == 'symbol' && !map.getLayer('points').getLayoutProperty('icon-image')) addMarkers({});
              })
            })
        }

        const addPoints = () => {
          // if link (could be more tricky to iterate points)
          //const geojson_node = document.getElementsByClassName("views-display-link-geojson_export_1").item(0)
          //map.getSource('points').setData(geojson_node.attributes.href.value)

          const geojson_points = document.getElementsByClassName("maplibre-canvas-block-geojson-data").item(0) && JSON.parse(document.getElementsByClassName("maplibre-canvas-block-geojson-data").item(0).outerText) || {}
          if (geojson_points.features  && geojson_points.features.length > 0) {
            if (!map.getSource('points')) {
              map.addSource('points', {
                'type': 'geojson',
                'data': geojson_node.attributes.href.value,
                'cluster': true,
                'clusterRadius': 40,
              });

            } else {
              map.getSource('points').setData(geojson_points)
            }
            map.getSource('points').setClusterOptions({cluster:true})
            if (map.getLayer('clusters')) {
              map.getLayer('clusters').source = 'points'
              map.getLayer('cluster-count').source = 'points'
            }

            map.fitBounds(setBounds(geojson_points), {padding: {top: 125, bottom: 125, left: 125, right: 125}});
            // Only add markers if no symbol type and icon-image defined on points layer
            if (map.getLayer('points').type == 'symbol' && !map.getLayer('points').getLayoutProperty('icon-image')) addMarkers({})
            /*map.flyTo({
                  center: geojson_points.features[0].geometry.coordinates
                });*/


          }
        }

        const setBounds = (geojson) => {
          const bounds = new maplibregl.LngLatBounds();
          geojson.features.forEach(function (feature) {
            bounds.extend(feature.geometry.coordinates);
          });
          return bounds
        }
        const addMarkers = (geojson) => {
          // To prevent multiple markers we build a raw cache to
          // hide or show the made html depending on query rendered.
          const newMarkers = {};
          const renderedFeatures = map.queryRenderedFeatures({
            layers: ['points'],
            filter: [
              'all', ['==', '$type', 'Point'], ['!has', 'cluster']
            ]
          });
           renderedFeatures.forEach(function (feature) {
             if (visibleMarkers[feature.properties.name]) {
               // if exists make it visible
               visibleMarkers[feature.properties.name].removeClassName('maplibregl-attrib-empty');
               newMarkers[feature.properties.name] = visibleMarkers[feature.properties.name]
               return;
             }
             if (feature.properties.html_marker) {
               // cook the html with html_marker property
               const el = document.createElement('div');
               el.className = 'marker'
               el.setAttribute('name', 'newm')

               // create the popup
               //debugger
               const popupContents = Drupal.maplibregl_popupContents(feature)
               const popup = new maplibregl.Popup({offset: 25}).setHTML(popupContents.html);


               el.innerHTML = feature.properties.html_marker;
               newMarkers[feature.properties.name] = visibleMarkers[feature.properties.name] = new maplibregl.Marker({element: el})
                 .setLngLat(feature.geometry.coordinates)
                 .setPopup(popup)
                 .addTo(map)
               newMarkers[feature.properties.name].getElement().addEventListener('click', () => console.log('click event'))
             } else {
               newMarkers[feature.properties.name] = visibleMarkers[feature.properties.name] = new maplibregl.Marker({})
                 .setLngLat(feature.geometry.coordinates)
                 .setPopup(popup)
                 .addTo(map)
               newMarkers[feature.properties.name].getElement().addEventListener('click', () => console.log('click event'))
             }
           });
           for (const id in visibleMarkers){
             // if is not in newmarkers but in visible hide it
             if(!newMarkers[id]) visibleMarkers[id].addClassName('maplibregl-attrib-empty')//remove()
           }
        }
        // Options for the observer (which mutations to observe)
        const config = { attributes: false, childList: true, subtree: false };
        // Callback function to execute when mutations are observed
        const callback = (mutationList, observer) => {
          for (const mutation of mutationList) {
            if (mutation.type === "childList") {
              addPoints()
            }
          }
        };

        // Create an observer instance linked to the callback function
        const observer = new MutationObserver(callback);

        // Start observing the target node for configured mutations
        observer.observe(targetNode, config);

      })
    }
  }
})(Drupal, once)
